package dev.guoping.app_info;

import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.text.TextUtils;

import java.io.IOException;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import dev.guoping.app_info.utils.DeviceId;
import dev.guoping.app_info.utils.AppInfoPluginUtils;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * AppInfoPlugin
 */
public class AppInfoPlugin implements MethodCallHandler {
    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "dev.guoping.app_info");
        channel.setMethodCallHandler(new AppInfoPlugin());
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("getUdid")) {
            result.success(getUdid());
        } else if (call.method.equals("getModel")) {
            result.success(android.os.Build.MODEL);
        } else if (call.method.equals("getAppId")) {
            result.success(AppInfoPluginUtils.getContext().getPackageName());
        } else if (call.method.equals("getAppVerName")) {
            result.success(getAppVersionName());
        } else if (call.method.equals("getAppVerCode")) {
            result.success(getAppVersionCode());
        } else if (call.method.equals("getAppChannel")) {
            if (!TextUtils.isEmpty(AppInfoPluginUtils.getAppChannel())) {
                result.success(AppInfoPluginUtils.getAppChannel());
                return;
            }
            result.success(getAppChannel());
        } else {
            result.notImplemented();
        }
    }

    private String getUdid() {
        return DeviceId.get();
    }

    private PackageInfo getPackageInfo() {
        try {
            return AppInfoPluginUtils.getContext().getPackageManager().getPackageInfo(AppInfoPluginUtils.getContext().getPackageName(), 0);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getAppVersionName() {
        PackageInfo info = getPackageInfo();
        return info == null ? "" : info.versionName;
    }

    private Double getAppVersionCode() {
        PackageInfo info = getPackageInfo();
        return (double) info.versionCode;
    }

    private String getAppChannel() {
        String tag = "app_channel_";
        ApplicationInfo appInfo = AppInfoPluginUtils.getContext().getApplicationInfo();
        String sourceDir = appInfo.sourceDir;
        String ret = "";
        ZipFile zipfile = null;
        try {
            zipfile = new ZipFile(sourceDir);
            Enumeration<?> entries = zipfile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry entry = ((ZipEntry) entries.nextElement());
                String entryName = entry.getName();
                if (entryName.contains(tag)) {
                    ret = entryName;
                    break;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (zipfile != null) {
                try {
                    zipfile.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        try {
            int splitIndex = ret.lastIndexOf("/");
            if (splitIndex >= 0) {
                ret = ret.substring(splitIndex + 1);
            }
        } catch (Exception e) {
            ret = e.toString();
        }
        if (!TextUtils.isEmpty(ret)) {
            return ret.substring(tag.length());
        }
        return "";
    }
}
