package dev.guoping.app_info.utils;

import android.os.Environment;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.RandomAccessFile;

/**
 * Created by guoping on 2017/8/4.
 */

public class FileUtil {

    private FileUtil() {
        throw new UnsupportedOperationException("FileUtil cannot be instantiated");
    }

    public static boolean sdcardWriteable() {
        return Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED);
    }

    public static boolean append(File file, String content) {
        return append(file, content.getBytes());
    }

    public static boolean write(File file, String content) {
        return write(file, content.getBytes());
    }

    public static String read(File file) {
        byte[] data = readBytes(file);
        if (data != null) {
            return new String(data);
        }
        return null;
    }

    public static boolean append(File file, byte[] data) {
        try {
            if (sdcardWriteable()) {
                if (!checkAndMkDirs(file)) {
                    return false;
                }
                RandomAccessFile raf = new RandomAccessFile(file, "rw");
                raf.seek(file.length());
                raf.write(data);
                raf.close();
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static boolean write(File file, byte[] data) {
        try {
            if (sdcardWriteable()) {
                if (!checkAndMkDirs(file)) {
                    return false;
                }
                FileOutputStream s = new FileOutputStream(file);
                s.write(data);
                s.flush();
                s.close();
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public static byte[] readBytes(File file) {
        try {
            if (sdcardWriteable()) {
                if (!file.exists()) {
                    return null;
                }
                FileInputStream s = new FileInputStream(file);
                byte[] b = new byte[s.available()];
                s.read(b);
                return b;
            } else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static boolean checkAndMkDirs(File file) {
        File parent = file.getParentFile();
        if (parent != null && !parent.exists()) {
            return parent.mkdirs();
        }
        return true;
    }

}
