package dev.guoping.app_info.utils.local_data;

import android.os.Environment;
import android.text.TextUtils;
import android.util.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executors;

import dev.guoping.app_info.utils.AppInfoPluginUtils;
import dev.guoping.app_info.utils.FileUtil;

public class SdData extends LocalData {

    private static Map<String, SdData> instances = new HashMap<>();

    public static SdData getInstance(String instanceKey) {
        if (!instances.containsKey(instanceKey)) {
            instances.put(instanceKey, new SdData(instanceKey));
        }
        return instances.get(instanceKey);
    }

    private Map<String, String> data;
    private File file;

    SdData(String instanceKey) {
        super(instanceKey);
    }

    @Override
    protected synchronized void init() {
        String path = String.format("Android/data/data/%s", AppInfoPluginUtils.getContext().getPackageName());
        File dir = new File(Environment.getExternalStorageDirectory(), path);
        file = new File(dir, instanceKey);
        data = new HashMap<>();

        String str = FileUtil.read(file);
        if (TextUtils.isEmpty(str)) {
            return;
        }
        try {
            StringReader reader = new StringReader(str);
            BufferedReader br = new BufferedReader(reader);
            String line;
            while ((line = br.readLine()) != null) {
                if (!TextUtils.isEmpty(line)) {
                    Log.d("CACHE", "line:" + line);
                    String[] a = line.split("\\$");
                    Log.d("CACHE", "a.length:" + a.length + " data:" + data);
                    data.put(decode(a[0]), decode(a[1]));
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    @Override
    public Map<String, ?> getAll() {
        return data;
    }

    @Override
    protected synchronized String get(String key, String def) {
        if (data.containsKey(key)) {
            return data.get(key);
        }
        return def;
    }

    @Override
    protected synchronized void put(String key, String value) {
        data.put(key, value);
        apply();
    }

    @Override
    protected synchronized void remove(String key) {
        if (data.containsKey(key)) {
            data.remove(key);
            apply();
        }
    }

    @Override
    protected synchronized void clear() {
        data.clear();
        apply();
    }

    private void apply() {
        Executors.newSingleThreadExecutor().submit(new Runnable() {
            @Override
            public void run() {
                StringBuilder sb = new StringBuilder();
                try {
                    synchronized (SdData.this) {
                        for (String key : data.keySet()) {
                            String value = data.get(key);
                            sb.append(String.format("%s$%s\n", encode(key), encode(value)));
                        }
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                FileUtil.write(file, sb.toString());
            }
        });
    }

    private static String encode(String str) {
        return str.replace("#", "#0").replace("$", "#1").replace("\n", "#2");
    }

    private static String decode(String str) {
        return str.replace("#2", "\n").replace("#1", "$").replace("#0", "#");
    }
}
