//
//  KeyChainUtil.m
//  app_info
//
//  Created by 郭平 on 2019/7/8.
//

#import "AppInfoKeyChainUtil.h"
#import <Foundation/Foundation.h>

@implementation AppInfoKeyChainUtil

+ (BOOL)saveItem:(NSString *)item withService:(NSString *)service key:(NSString *)key {
    CFErrorRef error = NULL;
    NSInteger ac = 0;
    if (@available(iOS 9.0, *)) {
        ac = kSecAccessControlOr;
    }
    SecAccessControlRef accessControl =
    SecAccessControlCreateWithFlags(
        kCFAllocatorDefault,
        kSecAttrAccessibleWhenUnlockedThisDeviceOnly, // 只允许本设备 未锁屏时访问
        ac, // kSecAccessControlUserPresence 通过锁屏密码或者Touch ID进行验证
        &error
    );
    if (error) {
        return NO;
    }
    NSDictionary *query = @{
        (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
        (__bridge id)kSecValueData : [item dataUsingEncoding:NSUTF8StringEncoding],
        (__bridge id)kSecAttrService : service,
        (__bridge id)kSecAttrAccount : key,
        (__bridge id)kSecAttrAccessControl : (__bridge id)accessControl,
        (__bridge id)kSecAttrSynchronizable : @NO // 不同步到 icloud
    };
    CFRelease(accessControl);
    int n = [self deleteItemWithService:service key:key];
    n = SecItemAdd((__bridge CFDictionaryRef)query, nil);
    return n == errSecSuccess;
}

+ (NSString *)getItemWithService:(NSString *)service key:(NSString *)key prompt:(NSString *)prompt {
    NSDictionary *query = @{
        (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
        (__bridge id)kSecReturnData : @YES,
        (__bridge id)kSecAttrService : service,
        (__bridge id)kSecAttrAccount : key,
        (__bridge id)kSecUseOperationPrompt : prompt,
    };
    CFTypeRef dataTypeRef = NULL;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)query, &dataTypeRef);
    NSString *pwd = nil;
    if (status == errSecSuccess) {
        pwd = [[NSString alloc] initWithData:(__bridge NSData * _Nonnull)(dataTypeRef) encoding:NSUTF8StringEncoding];
    }
    if (dataTypeRef) {
        CFRelease(dataTypeRef);
    }
    return pwd;
}

+ (NSArray<NSString *> *)getAllItemsWithService:(NSString *)service prompt:(NSString *)prompt {
    NSDictionary *query = @{
        (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
        (__bridge id)kSecReturnData : @YES,
        (__bridge id)kSecAttrService : service,
        (__bridge id)kSecMatchLimit : (__bridge id)kSecMatchLimitAll
    };
    CFArrayRef result = NULL;
    OSStatus status = SecItemCopyMatching((__bridge CFDictionaryRef)query, (CFTypeRef *)&result);
    NSLog(@"%@",result);
    if (status == errSecSuccess) {
        NSArray *array = (__bridge_transfer NSArray*)result;
        NSMutableArray *ret = [NSMutableArray arrayWithCapacity:array.count];
        for (NSData *data in array) {
            NSString *pwd = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
            if (pwd.length > 0) {
                [ret addObject:pwd];
            }
        }
        return ret.copy;
    }
    return nil;
}

+ (BOOL)deleteItemWithService:(NSString *)service key:(NSString *)key {
    NSDictionary *query = @{
        (__bridge id)kSecClass : (__bridge id)kSecClassGenericPassword,
        (__bridge id)kSecAttrService : service,
        (__bridge id)kSecAttrAccount : key,
    };
    OSStatus state = SecItemCopyMatching((__bridge CFDictionaryRef)query, NULL);
    if (state == errSecSuccess) {
        return SecItemDelete((__bridge CFDictionaryRef)query) == errSecSuccess;
    } else {
        return NO;
    }
}

@end

