#import <sys/utsname.h>
#import "AppInfoPlugin.h"
#import "AppInfoKeyChainUtil.h"

#define kService @"dev.guoping.app_info"
#define kAccount @"udid"

@implementation AppInfoPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"dev.guoping.app_info" binaryMessenger:[registrar messenger]];
    AppInfoPlugin* instance = [[AppInfoPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
        result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    } else if ([@"getUdid" isEqualToString:call.method]) {
        result([self udid]);
    } else if ([@"getModel" isEqualToString:call.method]) {
        result([self model]);
    } else if ([@"getAppId" isEqualToString:call.method]) {
        result([[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleIdentifier"]);
    } else if ([@"getAppVerName" isEqualToString:call.method]) {
        NSString *s = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
        if (!s) {
            s = @"";
        }
        result(s);
    } else if ([@"getAppVerCode" isEqualToString:call.method]) {
        NSString *s = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleVersion"];
        result(@(s.doubleValue));
    } else if ([@"getAppChannel" isEqualToString:call.method]) {
        if ([AppInfoPluginUtils appChannel]) {
            result([AppInfoPluginUtils appChannel]);
            return;
        }
        result(@"AppStore");
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (NSString *)model {
    struct utsname systemInfo;
    uname(&systemInfo);
    NSString *deviceModel = [NSString stringWithCString:systemInfo.machine encoding:NSUTF8StringEncoding];
    if (!deviceModel) {
        deviceModel = [UIDevice currentDevice].model;
    }
    return deviceModel;
}

- (NSString *)udid {
    NSString *udid = [AppInfoKeyChainUtil getItemWithService:kService key:kAccount prompt:@""];
    if (!udid) {
        udid = [[NSUUID UUID] UUIDString];
        [AppInfoKeyChainUtil saveItem:udid withService:kService key:kAccount];
    }
    return udid;
}

@end
